/* Splash screen */
SplashScreen {
    background: #000000;
    color: #ffffff;
    align: center middle;
}
SplashScreen > Static #splash-text {
    width: 70;
}

SplashScreen > Static #welcome {
    width: 70;
}

SplashScreen #welcome {
    content-align-horizontal: center;
    text-style: reverse;
    margin-bottom: 3;
}

SplashScreen #enigma-diagram {
    content-align-horizontal: center;
    margin-bottom: 3;
    margin-top: 3;
}

SplashScreen #any-key {
    margin-top: 3;
    content-align-horizontal: center;
}

SplashScreen #splash-text {
    margin-top: 3;
    text-align: center;
    content-align-horizontal: center;
}

/* Main screen */
MainScreen {
    background: #000000;
    color: #ffffff;
    align: center middle;
}

MainScreen > Static {
    width: 30;
}

/* Configure screen */

ConfigureScreen {
    background: #000000;
    color: #ffffff;
    
}

ConfigureScreen Static {
    margin-left: 2;
    align: center middle;
}

/* De/Encrypt screen */

EncryptScreen {
    background: #000000;
    color: #ffffff;
    
}

EncryptScreen Static {
    margin-left: 2;
    align: center middle;
}

EncryptScreen #enigma-diagram {
    content-align-horizontal: center;
    margin-top: 7;
    margin-left: 1;
}

EncryptScreen #enigma-wirings {
    content-align-horizontal: center;
    margin-top: 1;
    margin-bottom: 10;
    margin-right: 0;
}

EncryptScreen #enigma-diagram-vertical {
    content-align-horizontal: left;
    margin-top: 1;
    height: 200;
    /* width: 170*/
    
}

EncryptScreen #enigma-wirings-vertical {
    content-align-horizontal: left;
    margin-top: 0;
    margin-right: 1;
    height: 200;
    /*width: 40*/
    
}

EncryptScreen #enigma-diagram-wirings-horizontal {
    content-align-horizontal: left;
    margin-top: 1;
    margin-right: 1;
    height: 40;
    width: auto
}

EncryptScreen TextArea {
    content-align-horizontal: center;
    margin-right: 1;
    margin-left: 1;
    max-height: 4;
}

EncryptScreen #cleartext-label{
    margin-left: 2;
    margin-top: 1;
}

EncryptScreen #ciphertext-label{
    margin-left: 2;
    margin-top: 1;
}

/* Modals */

ExitConfiguration {
    align: center middle;
}

ExitConfiguration > Container {
    width: auto;
    height: auto;
    border: thick $background 99%;
    background: $surface;
    align: center middle;
}

ExitConfiguration > Container > Label {
    width: 100%;
    content-align-horizontal: center;
    margin-top: 1;
    margin-left: 1;
    margin-right: 1;
    text-align: center;
}

ExitConfiguration > Container > Horizontal {
    width: auto;
    height: auto;
}

ExitConfiguration > Container > Horizontal > Button {
    margin: 2 4;
}

ConfigurationNotComplete {
    align: center middle;
}

ConfigurationNotComplete > Container {
    width: 50;
    height: auto;
    border: thick $background 99%;
    background: $surface;
    align: center middle;
}

ConfigurationNotComplete > Container > Label {
    width: 100%;
    content-align-horizontal: center;
    margin-top: 1;
    margin-left: 1;
    margin-right: 1;
    text-align: center;
}

ConfigurationNotComplete > Container > Horizontal {
    width: auto;
    height: auto;
}

ConfigurationNotComplete > Container > Horizontal > Button {
    margin: 2 17;
}

ConfigurationNull {
    align: center middle;
}

ConfigurationNull > Container {
    width: 50;
    height: auto;
    border: thick $background 99%;
    background: $surface;
    align: center middle;
}

ConfigurationNull > Container > Label {
    width: 100%;
    content-align-horizontal: center;
    margin-top: 1;
    margin-left: 1;
    margin-right: 1;
    text-align: center;
}

ConfigurationNull > Container > Horizontal {
    width: auto;
    height: auto;
}

ConfigurationNull > Container > Horizontal > Button {
    margin: 2 17;
}

PlugboardContentInvalid {
    align: center middle;
}

PlugboardContentInvalid > Container {
    width: 50;
    height: auto;
    border: thick $background 99%;      
    background: $surface;
    align: center middle;
}

PlugboardContentInvalid > Container > Label {
    width: 100%;
    content-align-horizontal: center;
    margin-top: 1;
    margin-left: 1;
    margin-right: 1;
    text-align: center;
}

PlugboardContentInvalid > Container > Horizontal {
    width: auto;
    height: auto;
}

PlugboardContentInvalid > Container > Horizontal > Button {
    margin: 2 17;
}


/* commons */
.invisible {
    display: none;
}